@@ -40,7 +40,7 @@ import platform
 import sys
 import argparse
 from tqdm import tqdm
-import netCDF4 as nc
+import netCDF4 as nc            
 import numpy as np
 import arguments
 from datetime import datetime
@@ -56,6 +56,7 @@ import warnings
 from importlib.metadata import version, PackageNotFoundError
 import _version
 import matplotlib
+from concurrent.futures import ProcessPoolExecutor, as_completed
 matplotlib.use("agg")
 warnings.filterwarnings("ignore", module="matplotlib")
 warnings.filterwarnings("ignore", module="datetime")
@@ -78,7 +79,7 @@ class Wrfplot(object):
     """Main class to deal with WRF input data"""
 
     def __init__(
-        self, input_path, output_path=None, variables=[], ulevels=None, dpi=150, cmap=False, animation=False, animation_speed=0.5,
+        self, input_path, output_path=None, variables=[], ulevels=None, dpi=150, cmap=False, animation=False, animation_speed=0.5, clevels=False,
     ):
         self.input_file = input_path
         self.nc_fh = None
@@ -93,11 +94,12 @@ class Wrfplot(object):
         self.ulevels = ulevels
         self.ulevel = None
         self.proj = None
-        self.clevels = []
+        # self.clevels = []
         self.cycle = None
         self.file = fileio.FileIO(self.input_file)
         self.config = ConfigParser()
         self.config.read(os.path.join(utils.data_dir(), "wrf_variables.ini"))
+        self.wrf_var_dict = {s: dict(self.config.items(s)) for s in self.config.sections()} 
         self.dpi = dpi
         self.cmap = cmap
         self.U = None
@@ -106,6 +108,7 @@ class Wrfplot(object):
         self.rainnc = None
         self.animation = animation
         self.speed = animation_speed
+        self.clevels = clevels
 
     def read_file(self, input_path):
         """Read input NetCDF file
@@ -191,8 +194,14 @@ class Wrfplot(object):
             self.var_data = getvar(self.nc_fh, var, ALL_TIMES)
 
     def read_latlons(self):
-        """Extract lat & lon data and update ``self.lats`` and ``self.lons`` instance variables accordingly"""
-        self.lats, self.lons = latlon_coords(self.var_data)
+        """Extract lat & lon data and update ``self.lats`` and ``self.lons`` variables accordingly"""
+        lats, lons = latlon_coords(self.var_data)
+        if lats.ndim == 3:
+            self.lats = lats[0] 
+            self.lons = lons[0]
+        else:
+            self.lats = lats
+            self.lons = lons
 
     def plot_variables(self):
         """A wrapper function to redirect plotting to surface and upper air variables to respective function"""
@@ -224,40 +233,43 @@ class Wrfplot(object):
     def get_time_period(self):
         """Extract times of the WRF output file"""
         dates = self.var_data.Time
-        for _date in dates:
-            if not np.isnat(_date):
-                date_time = self.to_datetime(_date)
-                year = date_time.strftime("%Y")
-                month = date_time.strftime("%m")
-                day = date_time.strftime("%d")
-                _time = date_time.strftime("%H:%M")
-                self.date_time.append(day + "-" + month + "-" + year + "_" + _time)
+        if dates.size == 1:
+            date_time = self.to_datetime(dates)
+            year = date_time.strftime("%Y")
+            month = date_time.strftime("%m")
+            day = date_time.strftime("%d")
+            _time = date_time.strftime("%H:%M")
+            self.date_time.append(day + "-" + month + "-" + year + "_" + _time)
+        else:
+            for _date in dates:
+                if not np.isnat(_date):
+                    date_time = self.to_datetime(_date)
+                    year = date_time.strftime("%Y")
+                    month = date_time.strftime("%m")
+                    day = date_time.strftime("%d")
+                    _time = date_time.strftime("%H:%M")
+                    self.date_time.append(day + "-" + month + "-" + year + "_" + _time)
 
         if self.date_time:
             self.cycle = self.date_time[0]
 
     def plot_sfc(self):
         """Plot only surface data"""
-        pbar = tqdm(
-            range(0, len(self.date_time)),
-            desc="Plotting Variable : ",
-            leave=False,
-            colour="green",
-        )
-        pbar.write(
-            "\n*** Initialising plotting for variable : "
-            + utils.quote(self.var)
-            + " ***\n"
-        )
+        pbar = tqdm(range(0, len(self.date_time)), desc="Plotting Variable : ", leave=False, colour="green")
+        pbar.write("\n*** Initialising plotting for variable : {_var} ***\n".format(_var=utils.quote(self.var)))
         data_plot = None
         images = []
+        
         for index in pbar:
             _time = self.date_time[index]
             if self.var == "slp":
                 self.var_data = smooth2d(self.var_data, 3, cenweight=4)
                 self.clevels = utils.get_auto_clevel(self.var_data, slp=True)
-            else:
-                self.clevels = json.loads(self.config.get(self.var, "clevels"))
+            elif self.clevels is False:
+                default_clevels = self.config.get(self.var, "clevels")
+                if ',' in default_clevels:
+                    default_clevels = list(map(int, default_clevels.split(',')))
+                self.clevels = default_clevels      
 
             if data_plot is None:
                 data_plot = self.var_data
@@ -267,9 +279,7 @@ class Wrfplot(object):
             path = self.plot_wrf_data(_time, index, data_plot=data_plot)
             if path:
                 images.append(path)
-        tqdm.write(
-            "\nPlotting of variable " + utils.quote(self.var) + " completed...\n"
-        )
+        tqdm.write("\nPlotting of variable " + utils.quote(self.var) + " completed...\n")
         if self.animation is not False:
             if len(images) > 0:
                 output_gif_name = os.path.join(os.path.dirname(images[0]), self.var) + '.gif'
@@ -280,38 +290,24 @@ class Wrfplot(object):
     def plot_upper(self):
         """Plot only upper atmospheric data"""
         pressure = getvar(self.nc_fh, "pressure", ALL_TIMES)
-        self.clevels = json.loads(self.config.get(self.var, "clevels"))
+        # self.clevels = self.config.get(self.var, "clevels")
+        if self.clevels is False:
+            default_clevels = self.config.get(self.var, "clevels")
+            if ',' in default_clevels:
+                default_clevels = list(map(int, default_clevels.split(',')))
+            self.clevels = default_clevels
         u_var_data = self.var_data
         if self.U is not None and self.V is not None:
             u_var_u_data = self.U
             u_var_v_data = self.V
-        pbar_var = tqdm(
-            range(0, len(self.date_time)),
-            desc="Plotting Variable : ",
-            leave=False,
-            position=0,
-            colour="green",
-        )
-        pbar_var.write(
-            "\n*** Initialising plotting for variable : "
-            + utils.quote(self.var)
-            + " ***\n"
-        )
+        pbar_var = tqdm(range(0, len(self.date_time)), desc="Plotting Variable : ", leave=False, position=0, colour="green")
+        pbar_var.write("\n*** Initialising plotting for variable : {_var} ***\n".format(_var=utils.quote(self.var)))
         images = []
         for index in pbar_var:
             _time = self.date_time[index]
             # Must initialise the bar inside time loop so that range is set every time entering into outer loop
-            pbar_lvl = tqdm(
-                self.ulevels,
-                desc="Plotting Level : ",
-                leave=False,
-                position=1,
-                colour="blue",
-            )
-
-            pbar_var.write(
-                "\tPlotting " + utils.quote(self.var) + " for Time: " + _time
-            )
+            pbar_lvl = tqdm(self.ulevels, desc="Plotting Level : ", leave=False, position=1, colour="blue")
+            pbar_var.write("\tPlotting " + utils.quote(self.var) + " for Time: " + _time)
             pbar_var.set_description("Overall Progress :", refresh=True)
             for self.ulevel in pbar_lvl:
                 self.var_data = interplevel(u_var_data, pressure, self.ulevel)
@@ -333,6 +329,11 @@ class Wrfplot(object):
 
     def plot_wrf_data(self, _time, index, data_plot=None, level=None):
         """A wrapper function to call main plot method from plot class"""
+        # print(data_plot.size, data_plot.shape, data_plot.ndim)
+        # The wrfout data for single time will not have time. Hence required to extract the whole array. Make it automated here.
+        if data_plot.ndim == 2:
+            index = np.index_exp[:]
+        
         # For rainfall data we need to extract previous time data with current time data
         if self.var == "ppn":
             current = self.rainnc[index] + self.rainc[index]
@@ -399,11 +400,11 @@ class Wrfplot(object):
 
     def convert_unit(self, data):
         """Convert data to other unit"""
-        if self.var in ["T2", "u_temp", "u_theta", "u_tv", "u_twb"]:
+        if self.var in ["T2", "u_temp", "u_theta", "u_tv", "u_twb"]:                    
             return convert.k_to_c(data)
         elif self.var in ["low_cloudfrac", "mid_cloudfrac", "high_cloudfrac"]:
             return np.round(data * 100)
-        elif self.var == "winds":
+        elif self.var in ["winds", 'u_winds']:
             return convert.ms_to_kts(data)
         else:
             return data
@@ -414,7 +415,7 @@ class Wrfplot(object):
 
     def get_title(self, var):
         """Get title from config file"""
-        return json.loads(self.config.get(var, "title"))
+        return self.config.get(var, "title")
 
     def get_unit(self):
         """Get unit from config file"""
@@ -423,7 +424,12 @@ class Wrfplot(object):
     def get_proj(self):
         """Get projection details from data extracted"""
         if self.proj is None:
-            self.proj = get_cartopy(self.var_data)
+            if self.var_data.ndim == 3:
+                self.proj = get_cartopy(self.var_data[0])
+            elif self.var_data.ndim == 4:
+                self.proj = get_cartopy(self.var_data[0, 0])
+            else:
+                self.proj = get_cartopy(self.var_data)
 
 
 def _praser():
@@ -474,6 +480,13 @@ def _praser():
         default=False,
         help="Valid colormap name to fill colors. Use '--list-cmaps' option to see list of supported colormaps. Must have minimum 11 colors, else will lead to error.",
     )
+    parser.add_argument(
+        "--clevels",
+        metavar="<contour-levels>",
+        type=arguments.validate_clevels,
+        default=False,
+        help="Provide custom contour level(s) to highlight data. Levels are to be in ascending order and seperated by ',' i.e., '24,26,28'. If single value is provided, clevels will be automatically calculated.",
+    )
     parser.add_argument(
         "--dpi",
         metavar="<value>",
@@ -512,13 +525,13 @@ def main():
         sys.exit(print(__version__))
     elif not all([args.input, args.vars, args.output]):
         sys.exit(
-            "You must provide path to WRF model output file using '--input' option, output directory for saving image files using '--output' option and must provide at least one variable name using '--var' option to process.\nTypical usage will be \"wrfplot --input filename' --output 'path/to/output/dir' --var 'slp'\""
+            "You must provide path to WRF model output file using '--input' option, output directory for saving image files using '--output' option and must provide at least one variable name using '--vars' option to process.\nTypical usage will be \"wrfplot --input filename' --output 'path/to/output/dir' --vars 'slp'\""
         )
     elif all([args.input, args.vars, args.output]):
         file = fileio.FileIO(args.input)
         if file.is_wrf():
             wrfplt = Wrfplot(
-                input_path=args.input, output_path=args.output, dpi=args.dpi, cmap=args.cmap, ulevels=args.ulevels, animation=args.gif, animation_speed=args.gif_speed
+                input_path=args.input, output_path=args.output, dpi=args.dpi, cmap=args.cmap, ulevels=args.ulevels, animation=args.gif, animation_speed=args.gif_speed, clevels=args.clevels
             )
             try:
                 wrfplt.read_file(args.input)
